#!/usr/bin/env python

"""
Directory output context.

Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.output.common import Output
from moinformat.utils.directory import Directory
from os.path import extsep, join, split, splitext

class DirectoryOutput(Output):

    "A directory output context."

    name = "directory"

    def __init__(self, parameters=None):

        "Initialise the context with the given 'parameters'."

        if not parameters or not parameters.has_key("filename"):
            raise ValueError, parameters

        Output.__init__(self, parameters)
        self.dir = Directory(parameters["filename"])
        self.dir.ensure()

        self.index_name = self.parameters.get("index_name") or "index.html"
        self.page_suffix = self.parameters.get("page_suffix") or "%shtml" % extsep

    # Convenience methods.

    def get_filename(self, filename):

        """
        Return the full path of a file with the given 'filename' found within
        the directory. The full path is an absolute path.
        """

        return self.dir.get_filename(filename)

    # Name translation methods.

    def to_filename(self, pagename):

        "Return the filename corresponding to 'pagename'."

        return "%s%s" % (pagename, self.page_suffix)

    def to_pagename(self, filename):

        "Return the pagename corresponding to 'filename'."

        # Take the leafname as the pagename from an arbitrary filename, removing
        # any file extension.

        return splitext(split(filename)[-1])[0]

    def to_parent_filename(self, pagename):

        "Return the parent page filename corresponding to 'pagename'."

        return pagename

    # Serialisation methods.

    def can_write(self):

        "Return whether this context supports page writing."

        return True

    def writefile(self, text, filename, encoding=None):

        """
        Write 'text' to the file having the given 'filename'. If the
        optional 'encoding' is specified, override the general encoding.
        """

        return self.writepath(text, self.dir.get_filename(filename), encoding)

    def writepage(self, text, pagename, encoding=None):

        """
        Write 'text' to the file having the given 'pagename' and optional
        'encoding'. If 'parent' is specified and a true value, it indicates that
        the page is a parent of other pages.
        """

        dir = self.dir
        parent = self.parent(pagename)

        # The page may have a parent.

        if parent:
            parentfile = self.to_filename(parent)
            parentdir = self.to_parent_filename(parent)

            # Relocate any file for the parent to an index file within a page
            # directory.

            if dir.isfile(parentfile):
                parent_tmp = "%s.tmp" % parentfile
                dir.rename(parentfile, parent_tmp)
                if not dir.exists(parentdir):
                    dir.makedirs(parentdir)
                dir.rename(parent_tmp, join(parentdir, self.index_name))

            # Or make a directory for the parent.

            elif not dir.exists(parentdir):
                dir.makedirs(parentdir)

        # Write to an index filename within any existing directory.

        dirname = self.to_parent_filename(pagename)

        if dir.isdir(dirname):
            filename = join(dirname, self.index_name)
        else:
            filename = self.to_filename(pagename)

        self.writefile(text, filename, encoding)

output = DirectoryOutput

# vim: tabstop=4 expandtab shiftwidth=4
